angular.module('condition').controller("PropertyCondition", function ($scope, i18nService, profileService, definitionService) {
    var choices = [
        {
            "id": "equals",
            "name": "comparisonOperator.equals",
            "appliesTo": ["email", "string", "integer"]
        }, {
            "id": "notEquals",
            "name": "comparisonOperator.notEquals",
            "appliesTo": ["email", "string", "integer"]
        }, {
            "id": "lessThan",
            "name": "comparisonOperator.lessThan",
            "appliesTo": ["integer"]
        }, {
            "id": "greaterThan",
            "name": "comparisonOperator.greaterThan",
            "appliesTo": ["integer"]
        }, {
            "id": "lessThanOrEqualTo",
            "name": "comparisonOperator.lessThanOrEqualTo",
            "appliesTo": ["integer"]
        }, {
            "id": "greaterThanOrEqualTo",
            "name": "comparisonOperator.greaterThanOrEqualTo",
            "appliesTo": ["integer"]
        }, {
            "id": "lessThanOrEqualTo",
            "name": "comparisonOperator.before",
            "appliesTo": ["date"]
        }, {
            "id": "greaterThanOrEqualTo",
            "name": "comparisonOperator.after",
            "appliesTo": ["date"]
        }, {
            "id": "isDay",
            "name": "comparisonOperator.isDay",
            "appliesTo": ["date"]
        }, {
            "id": "isNotDay",
            "name": "comparisonOperator.isNotDay",
            "appliesTo": ["date"]
        }, {
            "id": "between",
            "name": "comparisonOperator.between",
            "appliesTo": ["date", "integer"]
        }, {
            "id": "startsWith",
            "name": "comparisonOperator.startsWith",
            "appliesTo": ["email", "string"]
        }, {
            "id": "endsWith",
            "name": "comparisonOperator.endsWith",
            "appliesTo": ["email", "string"]
        }, {
            "id": "matchesRegex",
            "name": "comparisonOperator.matchesRegex",
            "appliesTo": ["email", "string"]
        }, {
            "id": "contains",
            "name": "comparisonOperator.contains",
            "appliesTo": ["email", "string"]
        }, {
            "id": "exists",
            "name": "comparisonOperator.exists"
        }, {
            "id": "missing",
            "name": "comparisonOperator.missing"
        }, {
            "id": "in",
            "name": "comparisonOperator.in",
            "appliesTo": ["email", "string", "integer"]
        }, {
            "id": "notIn",
            "name": "comparisonOperator.notIn",
            "appliesTo": ["email", "string", "integer"]
        }, {
            "id": "all",
            "name": "comparisonOperator.all"
        }];

    function initComparisonOperators(valueType) {
        if (valueType) {
            found = false;
            var filtered = [];
            for (var i = 0; i < choices.length; i++) {
                var c = choices[i];
                if (valueType[1] && c.id != 'in' && c.id != 'notIn' && c.id != 'all'
                    || !valueType[1] && c.id == 'all'
                    || c.appliesTo && c.appliesTo.length > 0 && c.appliesTo.indexOf(valueType[0]) == -1) {
                    continue;
                }
                if ($scope.ref.parameterValues.comparisonOperator == c.id) {
                    found = true;
                }
                filtered.push(c);
            }
            $scope.propertyCondition.operators = filtered;
            if (!found) {
                $scope.ref.parameterValues.comparisonOperator = filtered[0].id;
            }
        } else {
            $scope.propertyCondition.operators = choices;
        }
    }

    function initPropertyValueType() {
        var pn = $scope.propertyCondition.selectedPropertyName;
        if (pn == null) {
            $scope.propertyCondition.propertyValueType = null;
        }
        var choices = $scope.propertyCondition.propertyNameChoiceListValues;
        if (choices) {
            for (var i = 0; i < choices.length; i++) {
                var c = choices[i];
                if (c.metadata.id == pn) {
                    $scope.propertyCondition.propertyValueType = [c.type, c.multivalued];
                    return;
                }
            }
        }
        $scope.propertyCondition.propertyValueType = null;
    }

    function initPropertyValueFieldType() {
        var op = $scope.ref.parameterValues.comparisonOperator;
        if (typeof op != 'undefined' && (op == 'exists' || op == 'missing')) {
            $scope.propertyCondition.propertyValueFieldType = 'none';
        } else {
            var vt = $scope.propertyCondition.propertyValueType;
            $scope.propertyCondition.propertyValueFieldType = vt != null ? vt[0] : 'string';
        }
        $scope.propertyCondition.multivaluePropertyField = (typeof op != 'undefined' && (op == 'in' || op == 'notIn' || op == 'all' || op == 'between'));
    }

    function resetPropertyValues() {
        delete $scope.ref.parameterValues.propertyValue;
        delete $scope.ref.parameterValues.propertyValueInteger;
        delete $scope.ref.parameterValues.propertyValueDate;
        delete $scope.ref.parameterValues.propertyValues;
        delete $scope.ref.parameterValues.propertyValuesInteger;
        delete $scope.ref.parameterValues.propertyValuesDate;
        delete $scope.ref.parameterValues.propertyValueDateExpr;
        delete $scope.ref.parameterValues.propertyValuesDateExpr;
        if ($scope.propertyCondition.multivaluePropertyField) {
            if ($scope.propertyCondition.propertyValueFieldType == 'date') {
                $scope.ref.parameterValues.propertyValuesDate = [undefined];
            } else if ($scope.propertyCondition.propertyValueFieldType == 'integer') {
                $scope.ref.parameterValues.propertyValuesInteger = [undefined];
            } else if ($scope.propertyCondition.propertyValueFieldType != 'none') {
                $scope.ref.parameterValues.propertyValues = [''];
            }
        } else {
            if ($scope.propertyCondition.propertyValueFieldType == 'date') {
                $scope.ref.parameterValues.propertyValueDate = undefined;
            } else if ($scope.propertyCondition.propertyValueFieldType == 'integer') {
                $scope.ref.parameterValues.propertyValueInteger = undefined;
            } else if ($scope.propertyCondition.propertyValueFieldType != 'none') {
                $scope.ref.parameterValues.propertyValue = '';
            }
        }
    }

    function switchMultiple() {
        if ($scope.propertyCondition.multivaluePropertyField) {
            if ($scope.propertyCondition.propertyValueFieldType == 'date') {
                $scope.ref.parameterValues.propertyValuesDate = [$scope.ref.parameterValues.propertyValueDate];
                delete $scope.ref.parameterValues.propertyValueDate;
            } else if ($scope.propertyCondition.propertyValueFieldType == 'integer') {
                $scope.ref.parameterValues.propertyValuesInteger = [$scope.ref.parameterValues.propertyValueInteger];
                delete $scope.ref.parameterValues.propertyValueInteger;
            } else if ($scope.propertyCondition.propertyValueFieldType != 'none') {
                $scope.ref.parameterValues.propertyValues = [$scope.ref.parameterValues.propertyValue];
                delete $scope.ref.parameterValues.propertyValue;
            }
        } else {
            if ($scope.propertyCondition.propertyValueFieldType == 'date') {
                $scope.ref.parameterValues.propertyValueDate = $scope.ref.parameterValues.propertyValuesDate[0];
                delete $scope.ref.parameterValues.propertyValuesDate;
            } else if ($scope.propertyCondition.propertyValueFieldType == 'integer') {
                $scope.ref.parameterValues.propertyValueInteger = $scope.ref.parameterValues.propertyValuesInteger[0];
                delete $scope.ref.parameterValues.propertyValuesInteger;
            } else if ($scope.propertyCondition.propertyValueFieldType != 'none') {
                $scope.ref.parameterValues.propertyValue = $scope.ref.parameterValues.propertyValues[0];
                delete $scope.ref.parameterValues.propertyValues;
            }
        }
    }

    $scope.propertyCondition = {};

    function processData(data) {
        propertyNameChoiceListValuesIds = [];
        propertyNameChoiceListValues = data;
        propertyNameChoiceListValues.forEach(function (prop) {
            prop.metadata.id = "properties." + prop.metadata.id;
            propertyNameChoiceListValuesIds.push(prop.metadata.id)
        });

        $scope.propertyCondition.propertyNameChoiceListValues = propertyNameChoiceListValues.slice(0);
        $scope.propertyCondition.propertyNameChoiceListValues.push({
            metadata: {
                id: 'user-specified-property-string',
                name: i18nService.message('condition.propertyCondition.othertext')
            },
            type: 'string',
            multivalued: false
        });
        $scope.propertyCondition.propertyNameChoiceListValues.push({
            metadata: {
                id: 'user-specified-property-integer',
                name: i18nService.message('condition.propertyCondition.othernumeric')
            },
            type: 'integer',
            multivalued: false
        });
        $scope.propertyCondition.propertyNameChoiceListValues.push({
            metadata: {
                id: 'user-specified-property-date',
                name: i18nService.message('condition.propertyCondition.otherdate')
            },
            type: 'date',
            multivalued: false
        });

        $scope.propertyCondition.predefinedProperty = $scope.ref.parameterValues.propertyName == null || propertyNameChoiceListValuesIds.indexOf($scope.ref.parameterValues.propertyName) > -1;
        if ($scope.propertyCondition.predefinedProperty) {
            $scope.propertyCondition.selectedPropertyName = $scope.ref.parameterValues.propertyName;
        } else {
            if ($scope.ref.parameterValues.propertyValueInteger || $scope.ref.parameterValues.propertyValuesInteger) {
                $scope.propertyCondition.selectedPropertyName = 'user-specified-property-integer';
            } else if ($scope.ref.parameterValues.propertyValueDate || $scope.ref.parameterValues.propertyValuesDate) {
                $scope.propertyCondition.selectedPropertyName = 'user-specified-property-date';
            } else {
                $scope.propertyCondition.selectedPropertyName = 'user-specified-property-string';
            }
        }

        initPropertyValueType();
        initPropertyValueFieldType();
        initComparisonOperators($scope.propertyCondition.propertyValueType);
    }

    if ($scope.conditionType.id == "profilePropertyCondition") {
        profileService.getPropertyTypesByTag("profileProperties", true).success(processData);
    } else if ($scope.conditionType.id == "sessionPropertyCondition") {
        profileService.getPropertyTypesByTag("sessionProperties", true).success(processData);
    } else {
        processData([]);
    }

    $scope.$watch("propertyCondition.selectedPropertyName", function (newValue, oldValue) {
        if (newValue != oldValue) {
            console.log("selectedPropertyName changed from " + oldValue + " to " + newValue);
            var selectedPropertyName = $scope.propertyCondition.selectedPropertyName;
            $scope.propertyCondition.predefinedProperty = selectedPropertyName != 'user-specified-property-string' && selectedPropertyName != 'user-specified-property-date' && selectedPropertyName != 'user-specified-property-integer';
            if ($scope.propertyCondition.predefinedProperty) {
                $scope.ref.parameterValues.propertyName = selectedPropertyName;
            }
            var oldPropertyValueType = $scope.propertyCondition.propertyValueType ? $scope.propertyCondition.propertyValueType[0] : undefined;
            initPropertyValueType();
            initPropertyValueFieldType();
            initComparisonOperators($scope.propertyCondition.propertyValueType);
            if ($scope.propertyCondition.propertyValueType && oldPropertyValueType != $scope.propertyCondition.propertyValueType[0]) {
                resetPropertyValues();
            }

        }
    });

    $scope.$watch("ref.parameterValues.comparisonOperator", function (newValue, oldValue) {
        if (newValue != oldValue) {
            console.log("comparisonOperator changed from " + oldValue + " to " + newValue);
            var oldMultivalue = $scope.propertyCondition.multivaluePropertyField;
            initPropertyValueFieldType();
            if ($scope.propertyCondition.propertyValueFieldType == 'none' || oldValue == 'exists' || oldValue == 'missing') {
                resetPropertyValues();
            } else if (oldMultivalue != $scope.propertyCondition.multivaluePropertyField) {
                switchMultiple();
            }
        }
    });

    $scope.requiredLabel = i18nService.message('label.required');
});
