angular.module('incomingSearches', ['wemServices', 'wemDirectives', 'ui.bootstrap', 'i18n', 'underscore', 'dtPager'])
    .run(['$rootScope', function ($rootScope) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };
    }])
    
    .controller('IncomingSearchesListCtrl', ['$scope', 'referrerService', 'i18nService', '$timeout', 'datepickerConfig', 'DTInstances', 'DTOptionsBuilder', 'DTColumnBuilder', '$compile', 'dataTableService', '$location', 'dtPagerService',
        function ($scope, referrerService, i18nService, $timeout, datepickerConfig, DTInstances, DTOptionsBuilder, DTColumnBuilder, $compile, dataTableService, $location, dtPagerService) {
            $scope.incomingSearchFrom = Date.now() + -30 * 24 * 3600 * 1000;
            $scope.incomingSearchTo = Date.now();

            var actionsHtml = function (data, type, full, meta) {
                var html = "<button title=\"" + i18nService.message("serverSettings.internalSearches.byReferrers") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"displaySearchByReffers(" + meta.row + ")\"><i class=\"material-icons\">http</i></button> ";
                html += "<button title=\"" + i18nService.message("serverSettings.incomingSearches.goToDetails") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"goToDetails(" + meta.row + ")\"><i class=\"material-icons\">list</i></button> ";
                return html;
            };

            var handleMissing = function (data, type, full, meta) {
                return ManagersUtils.humanReadableMissingIfNeeded(data, i18nService);
            };

            //No global controller where to put this function, it must be redefined for each app
            var rowCallback = function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                $compile(nRow)($scope);
            };

            var init = function (from, to) {
                var promise = referrerService.getQueries(from, to);
                dataTableService.setPromise(promise);
                $scope.dtOptions = dataTableService.generateDataTableOptions({
                    pageLength: 25,
                    defaultOrder: [[2, 'desc']],
                    rowCallback: rowCallback
                });

                var columns = [
                    {name: 'referrerQuery', bundleKey: 'serverSettings.incomingSearches.searchedTerms', actionsHtml: handleMissing},
                    {name: 'occurrences', bundleKey: 'serverSettings.incomingSearches.occurrences'},
                    {name: 'importance', bundleKey: 'serverSettings.incomingSearches.importance'},
                    {
                        name: null, bundleKey: 'serverSettings.manageWem.listAction',
                        actionsHtml: actionsHtml,
                        notSortable: true,
                        options: [
                            ["width", "180px"],
                            ["contentPadding", "0"]
                        ]
                    }
                ];
                $scope.dtColumns = dataTableService.generateDataTableColumns(columns);
                promise.then(function () {
                    dtPagerService.refreshPager($scope);
                });
            };

            init($scope.incomingSearchFrom, $scope.incomingSearchTo);

            // date config
            datepickerConfig.startingDay = 1;
            datepickerConfig.minMode = "day";
            datepickerConfig.datepickerMode = "day";
            datepickerConfig.showWeeks = false;
            $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
            $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
            $scope.datePickers = {};

            $scope.openDatePopup = function ($event, selector) {
                $event.preventDefault();
                $event.stopPropagation();

                angular.forEach($scope.datePickers, function (value, key) {
                    $scope.datePickers[key] = false;
                });
                $scope.datePickers[selector] = true;
            };

            $scope.setDtInstance = function (instance) {
                $scope.dtInstance = instance;
                ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
            };

            $scope.displaySearchByReffers = function (index) {
                var query = $scope.dtInstance.DataTable.row(index).data().referrerQuery;
                var encodedQuery = referrerService.selectQuery(query);
                $location.path('/incoming-search/by-referrers/' + encodedQuery);
            };

            $scope.goToDetails = function (index) {
                var query = $scope.dtInstance.DataTable.row(index).data().referrerQuery;
                var encodedQuery = referrerService.selectQuery(query);
                $location.path('/incoming-search/' + encodedQuery);
            };

            $scope.updateIncomingSearchesList = function () {
                init(new Date($scope.incomingSearchFrom).getTime(), new Date($scope.incomingSearchTo).getTime());
                $scope.dtInstance.dataTable.fnDraw();
            };

            $scope.getMessage = function (key) {
                return i18nService.message(key);
            };

        }])
    .controller('IncomingSearchDetails', ['$scope', '$routeParams', 'referrerService', 'i18nService', '$location', 'notificationService', 'DTInstances', 'DTOptionsBuilder', 'DTColumnBuilder', '$compile', 'dataTableService', 'dtPagerService',
        function ($scope, $routeParams, referrerService, i18nService, $location, notificationService, DTInstances, DTOptionsBuilder, DTColumnBuilder, $compile, dataTableService, dtPagerService) {
            var encodedQuery = $routeParams.queryID;
            $scope.incomingSearchQuery = ManagersUtils.humanReadableMissingIfNeeded(referrerService.decodeQuery(encodedQuery), i18nService);

            var columnsPlaceHolder = function (data, type, full, meta) {
                if (!data || data == '') {
                    return i18nService.message('serverSettings.internalSearches.missing');
                }
                if (meta.col == 1) {
                    return moment(data).locale(ManagersContext.jahiaUILocale).format('LLL');
                }
                if (meta.col == 4) {
                    return i18nService.message('profilesProperty.gender.' + data);
                }
                return data;
            };

            var serverData = function (sSource, aoData, fnCallback, oSettings) {
                var draw = _.find(aoData, function (x) {
                    return x.name == "draw"
                }).value;

                var offset = _.find(aoData, function (x) {
                    return x.name == "start"
                }).value;

                var length = _.find(aoData, function (x) {
                    return x.name == "length"
                }).value;

                var order = _.map(_.find(aoData, function (x) {
                    return x.name == "order"
                }).value, function (o) {
                    return ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData) + ":" + o.dir;
                }).join(",");

                referrerService.getQueryDetails(encodedQuery, offset, length, order).success(function(data) {
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": data.totalSize,
                        "iTotalDisplayRecords": data.totalSize,
                        "aaData": data.list
                    };
                    fnCallback(results);
                    dtPagerService.refreshPager($scope);
                })
                .error(function(data, status, headers, config) {
                    console.error(data);
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": 0,
                        "iTotalDisplayRecords": 0,
                        "aaData": []
                    };
                    fnCallback(results);
                    dtPagerService.refreshPager($scope);
                });
            };

            $scope.dtOptions = dataTableService.generateDataTableOptions({
                pageLength: 25,
                processing: true,
                serverSide: true,
                sDom: 't',
                defaultOrder: [[1, 'desc']],
                rowCallback: function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                    $compile(nRow)($scope);
                }
            });

            $scope.dtOptions.withFnServerData(serverData);

            var columns = [
                {name: 'properties.referrerHost', bundleKey: 'serverSettings.incomingSearches.incomingSearchDetails.referrers', actionsHtml: columnsPlaceHolder},
                {name: 'timeStamp', bundleKey: 'serverSettings.incomingSearches.date', actionsHtml: columnsPlaceHolder},
                {name: 'profile.properties.firstName', bundleKey: 'serverSettings.incomingSearches.firstName', actionsHtml: columnsPlaceHolder},
                {name: 'profile.properties.lastName', bundleKey: 'serverSettings.incomingSearches.lastName', actionsHtml: columnsPlaceHolder},
                {name: 'profile.properties.gender', bundleKey: 'serverSettings.incomingSearches.gender', actionsHtml: columnsPlaceHolder},
                {
                    name: null, bundleKey: 'serverSettings.incomingSearches.actions',
                    actionsHtml: function(data, type, full, meta) {
                        return '<button type="button" class="btn btn-icon command-dashboard" ng-click="goToEditProfile(\'' + data.profileId + '\')" title="' + i18nService.message('serverSettings.incomingSearches.editProfile') + '"><i class="material-icons">person</i></button>';
                    },
                    notSortable: true,
                    options: [
                        ["width", "90px"],
                        ["contentPadding", "0"]
                    ]
                }
            ];
            $scope.dtColumns = dataTableService.generateDataTableColumns(columns);

            $scope.setDtInstance = function (instance) {
                $scope.dtInstance = instance;
                ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
            };

            $scope.goToEditProfile = function (profileId) {
                $location.path('/incoming-search/' + encodedQuery + '/profile/' + profileId);
            };
        }])
    .controller('IncomingSearchByReferrers', ['$scope', '$routeParams', 'referrerService', 'i18nService', '$location', 'notificationService',
        function ($scope, $routeParams, referrerService, i18nService, $location, notificationService) {
            var encodedQuery = $routeParams.queryID;
            $scope.incomingSearchDetails = referrerService.getQueryReferrers(encodedQuery);
        }]);
