angular.module('manageMostVisited', ['i18n', 'wemServices', 'wemDirectives', 'wemModals', 'picker', 'dtPager'])
    .run(['$rootScope', function ($rootScope) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };
    }])
    .directive("mostvisitedWidget", [ '$q', '$compile', '$location', 'i18nService', 'dataTableService', '_', 'genericModal', 'internalSearchService',
        function ( $q, $compile, $location, i18nService, dataTableService, _, genericModal, internalSearchService) {
        return {
            restrict: "E",
            scope: {
                limit: '='
            },
            template: '<div ng-include="templateUrl"></div>',
            controller: ['$scope','$q', '$compile', '$location', 'i18nService', 'queryService', '_', 'DTOptionsBuilder', 'DTColumnBuilder', 'genericModal', 'internalSearchService',
                function ($scope, $q, $compile, $location, i18nService, queryService, _, DTOptionsBuilder, DTColumnBuilder, genericModal, internalSearchService) {

                dataTableService.setPromise(internalSearchService.getMostVisitedList($scope.limit));

                $scope.dtOptions = dataTableService.generateDataTableOptions(
                    {paging: false, ordering:false}
                );

                var columns = [
                    {name: 'url', bundleKey: 'serverSettings.mostVisitedPages.url'},
                    {name: 'occurrences', bundleKey: 'serverSettings.mostVisitedPages.occurrences'}
                ];
                $scope.dtColumns = dataTableService.generateDataTableColumns(columns);

                // resolve template
                $scope.templateUrl = ManagersContext.localTemplatesPath + "/mostvisited-pages/mostvisitedPages-widget.html";


            }]
        };
    }])

    .controller('ManageMostVisitedListCtrl', ['$scope', '$q', '$timeout', '$compile', '$location', 'datepickerConfig', 'i18nService', 'dataTableService', 'queryService', '_', 'genericModal', 'internalSearchService', 'dtPagerService',
        function ($scope, $q, $timeout, $compile, $location, datepickerConfig, i18nService, dataTableService, queryService, _, genericModal, internalSearchService, dtPagerService) {
            $scope.mostVisitedFrom = Date.now() + -30*24*3600*1000;
            $scope.mostVisitedTo = Date.now();
            
            var actionsHtml = function (data, type, full, meta) {
                return "<button title=\"" + i18nService.message("serverSettings.internalSearches.goToDetails") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"goToDetails('" + data.url + "')\"><span class=\"md md-searches\"></span></button> ";
            };

            //No global controller where to put this function, it must be redefined for each app
            var rowCallback = function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                $compile(nRow)($scope);
            };

            var init = function (from, to) {
                var promise = internalSearchService.getMostVisitedList(null, from, to);
                dataTableService.setPromise(promise);

                $scope.dtOptions = dataTableService.generateDataTableOptions(
                    {defaultOrder: [[1, 'desc']], rowCallback: rowCallback}
                );


                var columns = [
                    {name: 'url', bundleKey: 'serverSettings.mostVisitedPages.url'},
                    {name: 'occurrences', bundleKey: 'serverSettings.mostVisitedPages.occurrences'},
                    {name: null, bundleKey: 'serverSettings.manageWem.listAction',
                        actionsHtml: actionsHtml,
                        notSortable: true,
                        options: [
                            ["width", "180px"],
                            ["contentPadding", "0"]
                        ]
                    }
                ];

                $scope.dtColumns = dataTableService.generateDataTableColumns(columns);
                promise.then(function(){
                    dtPagerService.refreshPager($scope);
                });
            };

            init($scope.mostVisitedFrom, $scope.mostVisitedTo);

            // date config
            datepickerConfig.startingDay = 1;
            datepickerConfig.minMode = "day";
            datepickerConfig.datepickerMode = "day";
            datepickerConfig.showWeeks = false;
            $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
            $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
            $scope.datePickers = {};

            $scope.openDatePopup = function ($event, selector) {
                $event.preventDefault();
                $event.stopPropagation();

                angular.forEach($scope.datePickers, function(value, key) {
                    $scope.datePickers[key] = false;
                });
                $scope.datePickers[selector] = true;
            };

            $scope.setDtInstance = function (instance) {
                $scope.dtInstance = instance;
                ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
            };

            $scope.goToDetails = function (url) {
                internalSearchService.selectUrl(url);
                // the redirect to search details view using a fake route parameter so that we can properly find the previous page in profiles controllers
                $location.path('/mostvisited/selected');
            };

            $scope.updatePagesViewsList = function () {
                init(new Date($scope.mostVisitedFrom).getTime(), new Date($scope.mostVisitedTo).getTime());
                $scope.dtInstance.dataTable.fnDraw();
            };

            $scope.getMessage = function (key) {
                return i18nService.message(key);
            };

        }])

    .controller('MostVisitedCtrl', ['$scope', '$routeParams', 'internalSearchService', 'i18nService', '$location', 'notificationService',
        function ($scope, $routeParams, internalSearchService, i18nService, $location, notificationService) {

            $scope.searchDetail = internalSearchService.getMostVisitedDetails();

            $scope.getTabTitle = function (key) {
                return i18nService.message(key);
            };
        }]);