angular.module('managePersonalization', ['ngResource', 'condition', 'wemServices', 'wemDirectives', 'i18n', 'ui.bootstrap', 'picker'])

    .controller('ManagePersonalizationCtrl', ['$scope', '$resource', 'definitionService', 'i18nService', 'notificationService', 'segmentService',
        function ($scope, $resource, definitionService, i18nService, notificationService, segmentService) {
            var conditionTypesResource = $resource(ManagersContext.contextServerAdminUrl + "/cxs/definitions/conditions/tags/:tagId", {tagId: "profileCondition,sessionCondition"}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });
            var tagResource = $resource(ManagersContext.contextServerAdminUrl + "/cxs/definitions/tags/:tagId", {tagId: "profileTags"});

            var modes = {
                segments: {
                    "parameterValues": {
                        "segments": [],
                        "matchType": "all"
                    },
                    "type": "profileSegmentCondition"

                },
                location: {
                    "parameterValues": {},
                    "type": "geoLocationSessionCondition"

                },
                custom: {}
            };

            $scope.variants = variants;
            $scope.tabs = ["segments", "location", "custom"];

            $scope.toBeSaveVariants = {};
            angular.forEach($scope.variants, function (variant) {
                $scope.toBeSaveVariants[variant.id] = {
                    "id": variant.id,
                    "name": variant.name,
                    "systemName": variant.systemName,
                    "tab": variant.tab ? variant.tab : "segments",
                    "init": variant.tab && variant.jsonFilter,
                    "disableEdition": variant.tab != "custom",
                    "conditions": {}
                };

                angular.forEach($scope.tabs, function (tab) {
                    if (variant.tab == tab && variant.jsonFilter) {
                        $scope.toBeSaveVariants[variant.id].conditions[tab] = angular.copy(variant.jsonFilter);
                    } else {
                        $scope.toBeSaveVariants[variant.id].conditions[tab] = angular.copy(modes[tab]);
                    }
                });

                if (!$scope.toBeSaveVariant) {
                    $scope.toBeSaveVariant = $scope.toBeSaveVariants[variant.id];
                }
            });

            $scope.conditionTypes = [];
            $scope.tags = {};

            $scope.conditionTypes = conditionTypesResource.get({tagId: "profileCondition,sessionCondition"});
            $scope.eventConditionTypes = conditionTypesResource.get({tagId: "usableInPastEventCondition"});
            $scope.profileTags = tagResource.get({tagId: "profileTags"}, function (value, responseHeaders) {
                console.log("manage-personalization-conditions.js : tags loaded from context server.");
                angular.forEach(value.subTags, function (v) {
                    $scope.tags[v.id] = v
                });
                definitionService.fillTags("conditionTypes", $scope);
                definitionService.fillTags("eventConditionTypes", $scope);
            });

            $scope.validate = function (gwtValidation) {
                console.log("wem personalization: validating...");
                if ($scope.personalized && $scope.personalized.$invalid) {
                    $scope.personalized.showErrors = true;

                    var parsedErrors = ManagersUtils.parseErrorsInForm($scope.personalized);
                    if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                        var message = "";
                        angular.forEach(parsedErrors, function (value, key) {
                            message += ("- " + i18nService.message(key) + ". </br></br>")
                        });
                        gwtValidation.addValidationError(message.substring(0, message.length - "</br></br>".length));
                    }
                }
            };

            $scope.save = function (gwtNode) {
                angular.forEach($scope.toBeSaveVariants, function (variantToBeSave) {
                    var child = gwtNode.child(variantToBeSave.id, variantToBeSave.systemName);
                    if (variantToBeSave.init) {
                        segmentService.addDownloadMixin(variantToBeSave.conditions[variantToBeSave.tab]);
                        child.setProperty("wem:tab", variantToBeSave.tab, 1);
                        child.setProperty("wem:jsonFilter", angular.toJson(variantToBeSave.conditions[variantToBeSave.tab]), 1);
                    } else {
                        child.setProperty("wem:tab", "", 1);
                        child.setProperty("wem:jsonFilter", "", 1);
                    }
                });
            };

            $scope.selectVariant = function (selectedVariant) {
                //validate current condition if there is one
                if ($scope.toBeSaveVariant.init && $scope.personalized && $scope.personalized.$invalid) {
                    $scope.personalized.showErrors = true;

                    var parsedErrors = ManagersUtils.parseErrorsInForm($scope.personalized);

                    if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                        ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    }
                } else {
                    $scope.toBeSaveVariant = selectedVariant;
                }
            };

            $scope.hasError = function (propertyName) {
                return ManagersUtils.hasErrors($scope.personalized, $scope.personalized, propertyName);
            };

            $scope.selectTab = function (selectedTab) {
                $scope.toBeSaveVariant.tab = selectedTab;
                $scope.toBeSaveVariant.init = true;
                $scope.toBeSaveVariant.disableEdition = (selectedTab != "custom");
            };

            $scope.trimVariantName = function (variantName) {
                if (variantName.length > 25)
                    return variantName.substring(0, 25) + '...';
                else
                    return variantName;
            }
        }]);

