var pickerModule = angular.module('picker', ['ui.bootstrap']);

pickerModule.directive('checklistModel', ['$parse', '$compile', function ($parse, $compile) {
    // contains
    function contains(arr, item, comparator) {
        if (angular.isArray(arr)) {
            for (var i = arr.length; i--;) {
                if (comparator(arr[i], item)) {
                    return true;
                }
            }
        }
        return false;
    }

    // add
    function add(arr, item, comparator) {
        arr = angular.isArray(arr) ? arr : [];
        if (!contains(arr, item, comparator)) {
            arr.push(item);
        }
        return arr;
    }

    // remove
    function remove(arr, item, comparator) {
        if (angular.isArray(arr)) {
            for (var i = arr.length; i--;) {
                if (comparator(arr[i], item)) {
                    arr.splice(i, 1);
                    break;
                }
            }
        }
        return arr;
    }

    // http://stackoverflow.com/a/19228302/1458162
    function postLinkFn(scope, elem, attrs) {
        // compile with `ng-model` pointing to `checked`
        $compile(elem)(scope);

        // getter / setter for original model
        var getter = $parse(attrs.checklistModel);
        var setter = getter.assign;
        var checklistChange = $parse(attrs.checklistChange);

        // value added to list
        var value = $parse(attrs.checklistValue)(scope.$parent);


        var comparator = angular.equals;

        if (attrs.hasOwnProperty('checklistComparator')) {
            comparator = $parse(attrs.checklistComparator)(scope.$parent);
        }

        // watch UI checked change
        scope.$watch('checked', function (newValue, oldValue) {
            if (newValue === oldValue) {
                return;
            }
            var current = getter(scope.$parent);
            if (newValue === true) {
                setter(scope.$parent, add(current, value, comparator));
            } else {
                setter(scope.$parent, remove(current, value, comparator));
            }

            if (checklistChange) {
                checklistChange(scope);
            }
        });

        // declare one function to be used for both $watch functions
        function setChecked(newArr, oldArr) {
            scope.checked = contains(newArr, value, comparator);
        }

        // watch original model change
        // use the faster $watchCollection method if it's available
        if (angular.isFunction(scope.$parent.$watchCollection)) {
            scope.$parent.$watchCollection(attrs.checklistModel, setChecked);
        } else {
            scope.$parent.$watch(attrs.checklistModel, setChecked, true);
        }
    }

    return {
        restrict: 'A',
        priority: 1000,
        terminal: true,
        scope: true,
        compile: function (tElement, tAttrs) {
            if (tElement[0].tagName !== 'INPUT' || tAttrs.type !== 'checkbox') {
                throw 'checklist-model should be applied to `input[type="checkbox"]`.';
            }

            if (!tAttrs.checklistValue) {
                throw 'You should provide `checklist-value`.';
            }

            // exclude recursion
            tElement.removeAttr('checklist-model');

            // local scope var storing individual checkbox model
            tElement.attr('ng-model', 'checked');

            return postLinkFn;
        }
    };
}]);

pickerModule.directive("nodePicker", function () {
    return {
        restrict: "E",
        scope: {
            selectedPath: '=ngModel',
            selectedId: '=',
            nodeTypes: '@',
            selectableNodeTypes: '@',
            displayableNodesOnly: '@',
            basePath: '@',
            baseUrl: '@',
            label: '@'
        },
        link: function(scope, element, attrs) {
            if (!attrs.selectedId) {
                scope.selectedIdSpecified = false;
            } else {
                scope.selectedIdSpecified = true;
            }
        },
        templateUrl: ManagersContext.localTemplatesPath + "/picker/picker-popup.html",
        controller: function ($scope, $modal) {
            $scope.openPicker = function () {
                var modalInstance = $modal.open({
                    templateUrl: ManagersContext.localTemplatesPath + "/picker/node-picker.html",
                    controller: function ($scope, $modalInstance, directiveScope) {
                        $scope.root = {};
                        $scope.base = (directiveScope.baseUrl && ManagersContext[directiveScope.baseUrl]) ? ManagersContext[directiveScope.baseUrl] : ManagersContext.baseLive;

                        function selectInTree() {
                            if ($scope.selectedPath) {
                                $scope.root.children.forEach(function (child) {
                                    if ($scope.selectedPath != child.path && $scope.selectedPath.indexOf(child.path) == 0) {
                                        $scope.toggle(child, selectItemInTree)
                                    }
                                });
                            }
                        }

                        function selectItemInTree(item) {
                            if ($scope.selectedPath && item && item.hasChildren && item.children.length > 0) {
                                item.displayed = true;
                                item.children.forEach(function (child) {
                                    if ($scope.selectedPath != child.path && $scope.selectedPath.indexOf(child.path) == 0) {
                                        $scope.toggle(child, selectItemInTree)
                                    }
                                });
                            }
                        }

                        if (!directiveScope.basePath) {
                            directiveScope.basePath = "";
                        }

                        // init get root node
                        $.ajax({
                            type: "GET",
                            dataType: "json",
                            url: $scope.base + ManagersContext.currentSitePath + directiveScope.basePath + ".treeRootItem.json?nodeTypes=" + directiveScope.nodeTypes
                                 + (directiveScope.selectableNodeTypes ? "&selectableNodeTypes=" + directiveScope.selectableNodeTypes : "")
                                 + (directiveScope.displayableNodesOnly ? "&displayablenodeonly=true" : "")
                        }).done(function (data) {
                            $scope.$apply(function () {
                                if (data && data.length > 0) {
                                    $scope.root = data[0];
                                    $scope.root.displayed = true;
                                    $scope.$watch('selectedPath', selectInTree);
                                }
                            });
                        });

                        $scope.getIcon = function (isRoot, item) {
                            if (isRoot) {
                                return "fa fa-globe";
                            }

                            if (item) {
                                if ($scope.isSelectable(item)) {
                                    return "fa fa-file";
                                } else {
                                    return "fa fa-folder"
                                }
                            }

                            return "fa fa-file";
                        };

                        $scope.getArrow = function (item) {
                            if (item.hasChildren) {
                                if (item.displayed) {
                                    return "fa fa-chevron-down";
                                } else {
                                    return "fa fa-chevron-right";
                                }
                            }
                            return "";
                        };

                        $scope.getItemClass = function (item) {
                            if ($scope.isSelected(item)) {
                                return "selected";
                            }

                            if ($scope.isSelectable(item)) {
                                return "selectable";
                            }
                        };

                        $scope.isSelectable = function (item) {
                            return "selectable" == item.classes;
                        };

                        $scope.isSelected = function (item) {
                            return item.path == $scope.selectedPath;
                        };

                        $scope.select = function (item) {
                            if ("selectable" == item.classes) {
                                $modalInstance.close(item)
                            }
                        };

                        $scope.toggle = function (item, cb) {
                            if (item.hasChildren) {
                                if (item.children && item.children.length > 0) {
                                    item.displayed = !item.displayed;
                                    if (cb) {
                                        cb(item)
                                    }
                                } else {
                                    $.ajax({
                                        type: "GET",
                                        dataType: "json",
                                        url: $scope.base + item.path + ".treeRootItem.json?nodeTypes=" + directiveScope.nodeTypes
                                             + (directiveScope.selectableNodeTypes ? "&selectableNodeTypes=" + directiveScope.selectableNodeTypes : "")
                                             + (directiveScope.displayableNodesOnly ? "&displayablenodeonly=true" : "")
                                    }).done(function (data) {
                                        var children = data && data.length > 0 ? data[0].children : [];
                                        $scope.$apply(function () {
                                            item.children = children;
                                            item.displayed = true;
                                            if (cb) {
                                                cb(item)
                                            }
                                        });
                                    });
                                }
                            }
                        };
                    },
                    size: 'auto',
                    resolve: {
                        directiveScope: function () {
                            return $scope;
                        }
                    }
                });

                modalInstance.result.then(function (item) {
                    $scope.selectedPath = item.path;
                    if ($scope.selectedIdSpecified) {
                        $scope.selectedId = item.id;
                    }
                });

            }
        }
    };
});

pickerModule.directive('iframeOnload', [function () {
    return {
        scope: {
            callBack: '@iframeOnload'
        },
        link: function (scope, element, attrs) {
            element.on('load', function () {
                if (scope.$parent[scope.callBack]) {
                    scope.$parent[scope.callBack](element);
                }
            })
        }
    }
}]);

pickerModule.directive("htmlTagPicker", function () {
    return {
        restrict: "E",
        scope: {
            selectedTagId: '=ngModel',
            origin: '@',
            tag: '@',
            attrs: '@',
            displayAttr: '@',
            disabled: '@',
            label: '@',
            name: '@',
            required: '='
        },
        templateUrl: ManagersContext.localTemplatesPath + "/picker/html-tag-picker.html",
        controller: function ($scope) {
            $scope.tags = [];
            $scope.loaded = false;
            $scope.loading = false;
        },
        link: function (scope, element, attributes) {
            var tag = attributes.tag ? attributes.tag : "form";
            var attrs = attributes.attrs ? attributes.attrs.split('|') : ["name", "id"];
            var displayAttr = attributes.displayAttr;

            scope.toBoolean = function (value) {
                if (typeof value === 'function') {
                    value = true;
                } else if (value && value.length !== 0) {
                    var v = angular.lowercase("" + value);
                    value = !(v == 'f' || v == '0' || v == 'false' || v == 'no' || v == 'n' || v == '[]');
                } else {
                    value = false;
                }
                return value;
            };

            var displayIframe = function (src) {
                var $iframe = $("<iframe></iframe>");
                $iframe.css("display", "none");
                $iframe.on("load", function () {
                    var tags = [];
                    $iframe.contents().find(tag).each(function () {
                        var $element = $(this);
                        for (var i = 0; i < attrs.length; i++) {
                            var attr = attrs[i];
                            if ($element.is("[" + attr + "]")) {
                                var id = $element.attr(attr);
                                tags.push({
                                    "id": id,
                                    "display": (displayAttr && $element.is("[" + displayAttr + "]")) ? $element.attr(displayAttr) : id
                                });
                                break;
                            }
                        }
                    });
                    scope.$apply(function () {
                        scope.tags = tags;
                        scope.loaded = true;
                        scope.loading = false;
                    });
                });
                $iframe.attr("src", src);
                element.append($iframe);
            };

            attributes.$observe('origin', function (value) {
                if (value) {
                    scope.loaded = false;
                    scope.loading = true;
                    element.find("iframe").remove();
                    displayIframe(ManagersContext.baseLive + value + ".html?skipWem=true");
                }
            });
        }
    };
});

pickerModule.directive("htmlTagVisualPicker", function () {
    return {
        restrict: "E",
        scope: {
            selectedTagId: '=ngModel',
            origin: '@',
            tag: '@',
            attrs: '@',
            displayAttr: '@',
            disabled: '@',
            callback: '@'
        },
        templateUrl: ManagersContext.localTemplatesPath + "/picker/html-tag-visual-picker.html",
        controller: function ($scope, $modal) {
            $scope.showVisualPicker = function () {
                var modalInstance = $modal.open({
                    templateUrl: ManagersContext.localTemplatesPath + '/picker/html-tag-visual-picker-modal.html',
                    controller: 'htmlTagVisualPickerModal',
                    size: 'full',
                    windowClass: 'tagPickerModal',
                    backdrop: false,
                    resolve: {
                        options: function () {
                            return {
                                selectedTagId: $scope.selectedTagId,
                                src: ManagersContext.baseLive + $scope.origin + ".html?skipWem=true",
                                tag: $scope.tag,
                                attrs: $scope.attrs ? $scope.attrs.split('|') : ["name", "id"],
                                displayAttr: $scope.displayAttr,
                                disabled: $scope.disabled,
                                callback: $scope.callback
                            };
                        }
                    }
                });

                modalInstance.opened.then(function (e, u, o) {
                    $('.modal-content').css('height', $(window).height() * 0.8);
                });

                modalInstance.result.then(function (selectedTagId) {
                    $scope.selectedTagId = selectedTagId;
                });
            }
        }
    };
});

pickerModule.controller('htmlTagVisualPickerModal', function ($rootScope, $scope, $modalInstance, options) {
    $scope.options = options;

    $scope.ok = function () {
        $modalInstance.close($scope.options.selectedTagId);
        $('.modal-content').css('height', $('#newGoalForm').height());
    };

    $scope.iframeLoadedCallBack = function (element) {
        ["keypress","submit","focus","blur","tab"].forEach(function(eventName){
            frames['visualPickerFrame'].addEventListener(eventName, function(e){
                e.preventDefault();
                e.stopPropagation();

                // if an input is somehow focused, unfocus it
                var target = e.target;
                setTimeout(function(){
                    if (target.blur) target.blur();
                }, 50);
            }, true);
        });

        $('body', frames['visualPickerFrame'].document).bind("click", function (event) {
            var $targetElement = $(event.target);
            if ($scope.options.callback) {
                if ($rootScope[$scope.options.callback]) {
                    $rootScope[$scope.options.callback](event, $scope);
                }
            } else {

                var found = $targetElement.closest($scope.options.tag);
                if (found && found.length > 0) {
                    for (var i = 0; i < $scope.options.attrs.length; i++) {
                        var attr = $scope.options.attrs[i];
                        if (found.is("[" + attr + "]")) {
                            $scope.$apply(function () {
                                $scope.options.selectedTagId = found.attr(attr);
                            });
                            break;
                        }
                    }
                }
            }

            // disable links that trigger navigation on other pages, keep ajax links start with # and empty href links that can be used to display hided contents
            if(($targetElement.is("a") && $targetElement.attr("href") && $targetElement.attr("href").length > 0 && $targetElement.attr("href")[0] != "#")){
                event.preventDefault();
                event.stopPropagation();
                return false;
            }
        });
    };

    $scope.init = function () {
        //resize modal
        var modalContent = $('.modal-content').css('height', $(window).height() * 0.9);
        $('.modal-content iframe').css('height', modalContent.height() - 190); // 190 is the space needed of everything else than the modal core content
    };

    $scope.cancel = function () {
        $modalInstance.dismiss('cancel');
        $('.modal-content').css('height', $('#newGoalForm').height());
    };
});

pickerModule.directive("siteLanguagePicker", function () {
    return {
        restrict: "E",
        scope: {
            selectedLanguage: '=ngModel'
        },
        templateUrl: ManagersContext.localTemplatesPath + "/picker/site-languague-picker.html",
        controller: function ($scope) {
            $scope.root = {};
            $scope.loaded = false;

            $.ajax({
                type: "GET",
                dataType: "json",
                url: ManagersContext.baseLive + ManagersContext.currentSitePath + ".languages.json"
            }).done(function (data) {
                $scope.$apply(function () {
                    if (data && data.length > 0) {
                        $scope.siteLanguages = data;
                        $scope.loaded = true;
                    }
                });
            });

        }
    };
});

