angular.module('wemPersonaPanelApp', ['ngResource', 'ngCookies', 'uiGmapgoogle-maps', 'underscore', 'wemServices', 'wemDirectives', 'ui.bootstrap', 'ui-notification', 'i18n'])

    .config(function (uiGmapGoogleMapApiProvider) {
        uiGmapGoogleMapApiProvider.configure({
            key: ManagersContext.googleAPIKey,
            v: '3.17',
            libraries: 'weather,geometry,visualization'
        });
    })

    .controller('panelController', ['$scope', '_', 'profileService', '$timeout', '$cookies', 'i18nService', 'geonamesService', 'segmentService', function ($scope, _, profileService, $timeout, $cookies, i18nService, geonamesService, segmentService) {
        //Google map object
        $scope.map = ManagersUtils.personaMap;

        $scope.marker = {
            id: 0,
            coords: {
                latitude: 45,
                longitude: -73
            },
            options: {draggable: true},
            events: {
                dragend: function (marker, eventName, model, args) {
                    //Set new location
                    $scope.reverseGeoCode(args[0].latLng.lat(), args[0].latLng.lng());
                }
            }
        };

        if (sessionStorage.personaId) {
            $scope.selectedPersona = sessionStorage.personaId;
        } else {
            $scope.selectedPersona = "none";
        }

        //tooglable categories
        $scope.tooglable = {
            profile: true,
            interests: true,
            session: true,
            segments: true,
            location: true
        };

        $scope.locationHandler = {
            cityLoaded: false,
            countryLoaded: false,
            adm1Loaded: false,
            adm2Loaded: false
        };

        $scope.propertiesHandler = {
            segmentsToAdd: [],
            availableProfileProperties: [],
            availableSessionProperties: [],
            profilePropertyToAdd: null,
            sessionPropertyToAdd: null,
            interestToAdd: null,
            interestToAddValue: 1,
            profileListNotEmpty: false,
            sessionListNotEmpty: false,
            addProfileProperties: false,
            addSessionProperties: false,
            addInterest: false,
            addSegment: false
        };

        $scope.choiceListValues = {
            session : {
                deviceCategory : [
                    "Game console",
                    "Other",
                    "PDA",
                    "Personal computer",
                    "Smart TV",
                    "Smartphone",
                    "Tablet",
                    "Wearable computer"
                ]
            }
        };

        //Display size limit
        $scope.propertyNameLimit = 20;
        $scope.propertyValueLimit = 20;

        segmentService.getAll().success(function(segments) {
            $scope.allSegments = _.sortBy(segments, 'name');
        });
        profileService.getPropertyTypesByTag("profileProperties", true).success(function(propertyTypes) {
            $scope.availableProfileProperties= propertyTypes;
            $scope.availableProfilePropertiesIndexed= _.indexBy(propertyTypes, 'itemId');
        });
        profileService.getPropertyTypesByTag("sessionProperties", true).success(function(propertyTypes) {
            $scope.availableSessionProperties= propertyTypes;
            $scope.availableSessionPropertiesIndexed= _.indexBy(propertyTypes, 'itemId');
        });
        $scope.personaOverrides = angular.fromJson(sessionStorage.personaOverrides) || {};

        $scope.filterAvailableSegments = function (segment) {
            if($scope.profileSegments && $scope.profileSegments.length > 0){
                return !_.contains($scope.profileSegments, segment.id);
            }
            return true;
        };

        $scope.filterAvailableProps = function (property) {
            if (property.target == "profiles") {
                return $scope.profileProperties && !$scope.profileProperties.hasOwnProperty(property.itemId);
            } else {
                return $scope.sessionProperties && !$scope.sessionProperties.hasOwnProperty(property.itemId) &&
                    ['sessionCountryCode','sessionCity','sessionCityId','sessionCountryName','sessionAdminSubDiv1','sessionAdminSubDiv2','location'].indexOf(property.itemId) == -1
            }
        };

        $scope.filteredEditableProp = function (type) {
            if(type == "profile") {
                return _.filter(_.map($scope.profileProperties, function(value, name){ return name }), function(name) {
                    return $scope.availableProfilePropertiesIndexed.hasOwnProperty(name);
                });
            } else {
                return  _.filter(_.map($scope.sessionProperties, function(value, name){ return name }), function(name) {
                    return $scope.availableSessionPropertiesIndexed.hasOwnProperty(name) &&
                            ['sessionCountryCode','sessionCity','sessionCityId','sessionCountryName','sessionAdminSubDiv1','sessionAdminSubDiv2','location'].indexOf(name) == -1
                    });
            }
        };

        $scope.removeProp = function(name, type) {
            if(type == "profile") {
                if($scope.profileProperties.hasOwnProperty(name)) {
                    delete $scope.profileProperties[name];
                }
            } else if(type == "interest") {
                if($scope.profileProperties.interests && $scope.profileProperties.interests.hasOwnProperty(name)) {
                    delete $scope.profileProperties.interests[name];
                }
            } else {
                if($scope.sessionProperties.hasOwnProperty(name)) {
                    delete $scope.sessionProperties[name];
                }
            }
            $scope.updatePersonaOverride();
        };

        $scope.addProfileProp = function() {
            if($scope.propertiesHandler.profilePropertyToAdd) {
                if($scope.propertiesHandler.profilePropertyToAdd.type == "integer") {
                    $scope.profileProperties[$scope.propertiesHandler.profilePropertyToAdd.itemId] = 0;
                } else if($scope.propertiesHandler.profilePropertyToAdd.type == "date") {
                    $scope.profileProperties[$scope.propertiesHandler.profilePropertyToAdd.itemId] = new Date();
                } else {
                    $scope.profileProperties[$scope.propertiesHandler.profilePropertyToAdd.itemId] = "";
                }

                $scope.updatePersonaOverride();
                $scope.propertiesHandler.profilePropertyToAdd = null;
            }
        };

        $scope.addInterest = function () {
            if ($scope.propertiesHandler.interestToAdd) {
                if (!$scope.profileProperties.interests) {
                    $scope.profileProperties.interests = {};
                }
                $scope.profileProperties.interests[$scope.propertiesHandler.interestToAdd.toLowerCase()] = ($scope.propertiesHandler.interestToAddValue != undefined && $scope.propertiesHandler.interestToAddValue > 0) ? $scope.propertiesHandler.interestToAddValue : 1;

                $scope.updatePersonaOverride();
                $scope.propertiesHandler.interestToAdd = null;
                $scope.propertiesHandler.interestToAddValue = 1;
                $scope.propertiesHandler.addInterest = false;
            }
        };

        $scope.watchDate = function (name) {
            $scope.$watch(name, function(newVal, oldVal) {
                if(newVal && oldVal && Object.prototype.toString.call(newVal) === '[object Date]' && Object.prototype.toString.call(oldVal) === '[object Date]' && newVal.getTime() != oldVal.getTime()) {
                    $scope.updatePersonaOverride();
                }
            });
        };

        $scope.addSessionProp = function() {
            if($scope.propertiesHandler.sessionPropertyToAdd) {
                if($scope.propertiesHandler.sessionPropertyToAdd.type == "integer") {
                    $scope.sessionProperties[$scope.propertiesHandler.sessionPropertyToAdd.itemId] = 0;
                } else if($scope.propertiesHandler.sessionPropertyToAdd.type == "date") {
                    $scope.sessionProperties[$scope.propertiesHandler.sessionPropertyToAdd.itemId] = new Date();
                } else {
                    $scope.sessionProperties[$scope.propertiesHandler.sessionPropertyToAdd.itemId] = "";
                }

                $scope.updatePersonaOverride();
                $scope.propertiesHandler.sessionPropertyToAdd = null;
            }
        };

        $scope.addSegments = function () {
            angular.forEach($scope.propertiesHandler.segmentsToAdd, function (segment) {
                $scope.profileSegments.push(segment.id);
            });
            $scope.updatePersonaOverride();
            $scope.propertiesHandler.addSegment = false;
            $scope.propertiesHandler.segmentsToAdd = [];
        };

        $scope.removeSegment = function (segmentName) {
            $scope.profileSegments = _.filter($scope.profileSegments, function (segName) {
                return segName != segmentName;
            });
            $scope.updatePersonaOverride();
        };

        $scope.getSegmentName = function (segmentId) {
            var selectedSegment = _.find($scope.allSegments, function (segment) {
                return segment.id == segmentId;
            });
            if (selectedSegment) {
                return selectedSegment.name;
            }
            else {
                return segmentId;
            }
        };

        $scope.tooglableClick = function (toogleName) {
            $scope.tooglable[toogleName] = !$scope.tooglable[toogleName];
        };

        $scope.getPropertyInputType = function (propertyName, type) {
            var property = type == "profile" ? $scope.availableProfilePropertiesIndexed[propertyName] : $scope.availableSessionPropertiesIndexed[propertyName];
            if(!property) {
                return "text";
            }
            if(property.type == "integer") {
                return "number";
            } else if (property.type == "email") {
                return "email";
            } else if (property.type == "date") {
                return "date";
            }
            return "text";
        };

        $scope.getPropertyName = function(type, name) {
            if(type == "profile") {
                return ($scope.availableProfilePropertiesIndexed && $scope.availableProfilePropertiesIndexed[name]) ? $scope.availableProfilePropertiesIndexed[name].metadata.name : name;
            } else {
                return ($scope.availableSessionPropertiesIndexed && $scope.availableSessionPropertiesIndexed[name]) ? $scope.availableSessionPropertiesIndexed[name].metadata.name : name;
            }
        };

        /**
         * This function gets the full country list from the context server
         */
        $scope.getCountries = function () {
            geonamesService.getEntries().success(
                function (response) {
                    //Order list by name before returning it
                    $scope.countries = _.sortBy(response.list, 'name');
                    $scope.locationHandler.countryLoaded = true;
                }
            );
        };

        /**
         * This function gets the full country list from the context server
         */
        $scope.getAdm1 = function (countryCode, callback) {
            geonamesService.getEntries(countryCode).success(
                function (response) {
                    //Order list by name before returning it
                    if (_.size(response.list) > 0) {
                        if (response.list[0].featureClass == "P") {
                            $scope.adm1List = null;
                        }
                        else {
                            $scope.adm1List = _.sortBy(response.list, 'name');
                        }
                    }
                    else {
                        $scope.adm1List = null;
                    }
                    $scope.locationHandler.adm1Loaded = true;
                    callback();
                }
            );
        };
        /**
         * This function gets the full country list from the context server
         */
        $scope.getAdm2 = function (countryCode, adm1Id, callback) {
            var adm1Code = _.find($scope.adm1List, function (adm) {
                return adm.itemId == adm1Id;
            }).admin1Code;
            geonamesService.getEntries(countryCode, adm1Code).success(
                function (response) {
                    //Order list by name before returning it
                    if (_.size(response.list) > 0) {
                        if (response.list[0].featureClass == "P") {
                            $scope.adm2List = null;
                        }
                        else {
                            $scope.adm2List = _.sortBy(response.list, 'name');
                        }
                    }
                    else {
                        $scope.adm2List = null;
                    }
                    $scope.locationHandler.adm2Loaded = true;
                    if (callback != undefined) {
                        callback();
                    }
                }
            );
        };
        /**
         * This function gets the full country list from the context server
         */
        $scope.getCities = function (countryCode, admin1Id, admin2Id) {
            var oAdm1 = _.find($scope.adm1List, function (adm1) {
                return adm1.itemId == admin1Id;
            });
            var oAdm2 = _.find($scope.adm2List, function (adm2) {
                return adm2.itemId == admin2Id;
            });
            if (oAdm1) {
                var adm1Code = oAdm1.admin1Code;
            }
            if (oAdm2) {
                var adm2Code = oAdm2.admin2Code;
            }
            geonamesService.getCities(countryCode, adm1Code, adm2Code).success(function (response) {
                //Order list by name before returning it
                $scope.cities = _.sortBy(response.list, 'name');
                $scope.locationHandler.cityLoaded = true;
            });
        };

        $scope.updateLocation = function (location, cityName, cityId, countryCode, countryName, sessionAdminSubDiv1, sessionAdminSubDiv2) {
            //Update scope properties
            if(!$scope.sessionProperties.location) {
                $scope.sessionProperties.location = {};
            }
            $scope.sessionProperties.location.lat = location.lat;
            $scope.sessionProperties.location.lon = location.lon;
            $scope.sessionProperties.sessionCity = cityName;
            $scope.sessionProperties.sessionCityId = cityId;
            $scope.sessionProperties.sessionCountryCode = countryCode;
            $scope.sessionProperties.sessionCountryName = countryName;
            $scope.sessionProperties.sessionAdminSubDiv1 = sessionAdminSubDiv1;
            $scope.sessionProperties.sessionAdminSubDiv2 = sessionAdminSubDiv2;
            $scope.updatePersonaOverride();

            //Updating the google map marker
            $scope.marker.coords = {
                latitude: location.lat,
                longitude: location.lon
            };

            //Refresh google map
            $scope.map.control.refresh({latitude: location.lat, longitude: location.lon});
        };

        /**
         * This function gets the address of a point defined with latitude and longitude
         * @param lat : latitude
         * @param lon : longitude
         */
        $scope.reverseGeoCode = function (lat, lon) {
            //Reverse geocode resource url
            geonamesService.reverseGeoCode(lat, lon)
                .success(function (response) {
                    /* Found the address of the location*/
                    //Getting the proper city object from response
                    var city = _.find(response, function (subdiv) {
                        return subdiv.featureCode.indexOf("PPL") > -1;
                    });
                    //Getting the proper country object from response
                    var country = _.find(response, function (subdiv) {
                        return subdiv.featureCode.indexOf("PCL") > -1;
                    });
                    var admSub1 = _.find(response, function (subdiv) {
                        return subdiv.featureCode == "ADM1";
                    });

                    var admSub2 = _.find(response, function (subdiv) {
                        return subdiv.featureCode == "ADM2";
                    });
                    if (admSub1) {
                        var sessionAdminSubDiv1 = admSub1.itemId;
                    }
                    if (admSub2) {
                        var sessionAdminSubDiv2 = admSub2.itemId;
                    }
                    //We need at least to change the country to update the properties
                    if (country) {
                        $scope.refreshLocation(city, country.countryCode, country.name, sessionAdminSubDiv1, sessionAdminSubDiv2);
                        //$scope.updateLocation({lat:lat,lon:lon},city.name,city.itemId,country.countryCode,country.name,sessionAdminSubDiv1,sessionAdminSubDiv2)
                    }
                });
        };

        /**
         * Update location in scope, session storage and refresh the google map marker and center
         * @param city : geoname object
         * @param countryCode : String
         * @param countryName : String
         * @param sessionAdminSubDiv1 : String
         * @param sessionAdminSubDiv2 : String
         */
        $scope.refreshLocation = function (city, countryCode, countryName, sessionAdminSubDiv1, sessionAdminSubDiv2, propName) {
            var locationProperties = {
                location: {},
                sessionCity: null,
                sessionCityId: null,
                sessionAdminSubDiv2: sessionAdminSubDiv2,
                sessionAdminSubDiv1: sessionAdminSubDiv1,
                sessionCountryCode: countryCode,
                sessionCountryName: countryName
            };
            if (city) {
                locationProperties.location = {lat: city.location.lat, lon: city.location.lon};
                locationProperties.sessionCity = city.name;
                locationProperties.sessionCityId = city.itemId;
            }
            else {
                var oCountry = _.find($scope.countries, function (country) {
                    return country.countryCode == countryCode;
                });
                locationProperties.location = {lat: oCountry.location.lat, lon: oCountry.location.lon};
            }
            //Refresh geo lists
            if (countryCode) {
                //If country updated,then update adm1, adm2 & country list if needed
                if (propName == 'sessionCountryName') {
                    if (!sessionAdminSubDiv1) {
                        delete $scope.adm1List;
                        delete $scope.adm2List;
                        $scope.getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                    } else {
                        $scope.getAdm1(countryCode, function () {
                            if (sessionAdminSubDiv2) {
                                $scope.getAdm2(countryCode, sessionAdminSubDiv1, function () {
                                    $scope.getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                                });
                            }
                            else {
                                delete $scope.adm2List;
                                $scope.getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                            }
                        });
                    }
                }

                else if (propName == 'sessionAdminSubDiv1') {
                    if (!sessionAdminSubDiv2) {
                        $scope.getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                    } else {
                        $scope.getAdm2(countryCode, sessionAdminSubDiv1, function () {
                            $scope.getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                        });
                    }
                }

                else if (propName == 'sessionAdminSubDiv2') {
                    $scope.getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                }
                else {
                    if (countryName) {
                        if (!sessionAdminSubDiv1) {
                            delete $scope.adm1List;
                            delete $scope.adm2List;
                            $scope.getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                        } else {
                            $scope.getAdm1(countryCode, function () {
                                if (sessionAdminSubDiv2) {
                                    $scope.getAdm2(countryCode, sessionAdminSubDiv1, function () {
                                        $scope.getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                                    });
                                }
                                else {
                                    delete $scope.adm2List;
                                    $scope.getCities(countryCode, sessionAdminSubDiv1, sessionAdminSubDiv2);
                                }
                            });
                        }
                    }
                }
            }
            $scope.updateLocation(locationProperties.location, locationProperties.sessionCity, locationProperties.sessionCityId, locationProperties.sessionCountryCode, locationProperties.sessionCountryName, locationProperties.sessionAdminSubDiv1, locationProperties.sessionAdminSubDiv2);
        };

        /**
         * This function get the city to point the map on according to the given object and refreshes the map and the admin1, admin2 and cities lists
         * @param object : geoname object
         * @param type : String
         */
        $scope.setLocation = function (object, propName) {
            var sessionAdminSubDiv1 = null;
            var sessionAdminSubDiv2 = null;
            var countryCode = null;
            var countryName = null;
            //Get the capital of the entity and set location parameters
            if (propName != "sessionCityId") {
                geonamesService.getCapital(object.itemId)
                    .success(function (response) {
                        //City feature code must contain the String "PPL"
                        var city = _.find(response, function (citySubDiv) {
                            return citySubDiv.featureCode.indexOf("PPL") > -1;
                        });
                        //feature codes for admin sub areas are exactly "ADM1" and "ADM2"
                        var admSub1 = _.find(response, function (citySubDiv) {
                            return citySubDiv.featureCode == "ADM1";
                        });
                        var admSub2 = _.find(response, function (citySubDiv) {
                            return citySubDiv.featureCode == "ADM2";
                        });
                        //Feature code for country must contain the string "PCL"
                        var country = _.find(response, function (citySubDiv) {
                            return citySubDiv.featureCode.indexOf("PCL") > -1;
                        });
                        //If we found a location we refresh the scope and the map with it
                        if (city) {
                            countryCode = country.countryCode;
                            countryName = country.name;
                            //Update country code in scope
                            //Getting the sub area 1 from the response
                            if (admSub1) {
                                sessionAdminSubDiv1 = admSub1.itemId;
                            }
                            if (admSub2) {
                                sessionAdminSubDiv2 = admSub2.itemId;
                            }
                        } else {
                            //if we didn't find a location we base our refresh on the country contained in the object
                            countryCode = object.countryCode;
                            countryName = object.name;
                        }
                        $scope.refreshLocation(city, countryCode, countryName, sessionAdminSubDiv1, sessionAdminSubDiv2, propName);
                    }).error(function (response) {
                        //The capital city was not found
                        //reverse geocoding to get the cityname of the country center point
                        $scope.reverseGeoCode(object.location.lat, object.location.lon);
                    });
            } else {
                countryCode = $scope.sessionProperties.sessionCountryCode;
                countryName = $scope.sessionProperties.sessionCountryName;
                sessionAdminSubDiv1 = $scope.sessionProperties.sessionAdminSubDiv1;
                sessionAdminSubDiv2 = $scope.sessionProperties.sessionAdminSubDiv2;
                $scope.refreshLocation(object, countryCode, countryName, sessionAdminSubDiv1, sessionAdminSubDiv2, propName);
            }
        };

        /**
         * Update all address and location field when user changes country
         * Then save them to Browser session and refresh google Map
         * @param countryName
         */
        $scope.locationEdited = function (propName) {
            var locationValue = $scope.sessionProperties[propName];
            switch (propName) {
                case "sessionCountryName" :
                    //Get country from country name
                    var locationObject = _.find($scope.countries, function (scopeCountry) {
                        return scopeCountry.name == locationValue;
                    });
                    break;
                case "sessionAdminSubDiv1" :
                    var locationObject = _.find($scope.adm1List, function (oAdm) {
                        return oAdm.itemId == locationValue;
                    });
                    break;
                case "sessionAdminSubDiv2" :
                    var locationObject = _.find($scope.adm2List, function (oAdm) {
                        return oAdm.itemId == locationValue;
                    });
                    break;
                case "sessionCityId" :
                    var locationObject = _.find($scope.cities, function (oAdm) {
                        return oAdm.itemId == locationValue;
                    });
                    break;
            }
            //Get capital city and update adm1 and adm2 if needed
            $scope.setLocation(locationObject, propName);
        };


        /**
         * This function is called each time a foldable input send a blur event
         * It gets the property type (profiel or session) and save it to the browser session scope.
         * @param event
         */
        $scope.blurCallback = function (event) {
            $scope.updatePersonaOverride();
        };

        //Get modified persona
        $scope.hasOverridePersona = function() {
            return $scope.personaOverrides[$scope.selectedPersona];
        };

        $scope.getPersonaOverride = function () {
            var personaOverride = $scope.personaOverrides[$scope.selectedPersona];
            if (personaOverride == null) {
                personaOverride = {};
                personaOverride.personaId = $scope.selectedPersona;
            }
            return personaOverride;
        };

        //Save selected persona
        $scope.personaSelected = function () {
            var personaOverride = $scope.getPersonaOverride();
            $scope.propertiesHandler.addProfileProperties=false;
            $scope.propertiesHandler.addSessionProperties=false;
            if ($scope.selectedPersona != 'none') {
                window.parent.wem.loadPersonaContext($scope.selectedPersona, personaOverride.segments, personaOverride.profileProperties, personaOverride.sessionProperties);
            } else {
                window.parent.wem.loadContext(true, true);
            }
            sessionStorage.personaId=$scope.selectedPersona;
        };

        $scope.updatePersonaOverride = function () {
            var personaOverride = $scope.getPersonaOverride();

            personaOverride.profileProperties = angular.copy($scope.profileProperties);
            personaOverride.sessionProperties = angular.copy($scope.sessionProperties);
            personaOverride.segments = angular.copy($scope.profileSegments);

            $scope.personaOverrides[$scope.selectedPersona] = personaOverride;
            sessionStorage.personaOverrides = angular.toJson($scope.personaOverrides);

            $scope.propertiesHandler.addProfileProperties = false;
            $scope.propertiesHandler.addSessionProperties = false;
            window.parent.wem.loadPersonaContext($scope.selectedPersona, personaOverride.segments, personaOverride.profileProperties, personaOverride.sessionProperties);
            sessionStorage.personaId = $scope.selectedPersona;
        };

        /**
         * This function gets the properties labels starting from resource bundle or returning label if nothing is found there
         * @param category (profile or session)
         * @param propName : The property name
         * @returns An object containing the labels available in resource bundle for available properties
         */
        $scope.getPropertyResourceName = function (category, propName) {
            var label = i18nService.message(category + "Properties." + propName);
            if (label.indexOf("???") == -1) {
                return label;
            }
            return $scope.getPropertyName(category, propName);
        };

        /**
         *This function is called to reset the persona properties and segment to the values stored in the context server
         */
        $scope.resetPersona = function () {
            delete sessionStorage.personaOverrides;
            $scope.personaOverrides = {};
            window.parent.wem.loadPersonaContext($scope.selectedPersona, null, null, null);
            $scope.map.control.refresh($scope.sessionProperties.location.lat, $scope.sessionProperties.location.lon);
        };

        function contextLoaded(digitalData) {
            $timeout(function () {
                //Getting personaId from brower session
                var personaId = sessionStorage.personaId;
                if (personaId) {
                    $scope.selectedPersona = personaId;
                } else {
                    $scope.selectedPersona = "none";
                }
                $scope.profileId = window.parent.cxs.profileId;
                if (!$scope.hasOverridePersona()) {
                    $scope.profileProperties = window.parent.cxs.profileProperties;
                    $scope.sessionProperties = window.parent.cxs.sessionProperties;
                    $scope.profileSegments = window.parent.cxs.profileSegments;
                }

                $scope.filterTargets = window.parent.wem.getFilterTargets();
                $scope.isFilterOnTarget = function (target, value) {
                    for (var i = 0; i < $scope.filterTargets.length; i++) {
                        if ($scope.filterTargets[i].target == target) {
                            if (value) {
                                if ($scope.filterTargets[i].values.indexOf(value) > -1) {
                                    return true;
                                } else {
                                    return false;
                                }
                            } else {
                                return true;
                            }
                        }
                    }
                    return false;
                };

                if (!$scope.profileProperties) {
                    $scope.profileProperties = {}
                }
                if (!$scope.profileProperties.interests) {
                    $scope.profileProperties.interests = {}
                }
                if (!$scope.sessionProperties) {
                    $scope.sessionProperties = {}
                }

                // now let's copy the overrides into the currently loaded segment, profile and session properties.
                var personaOverride = $scope.getPersonaOverride();
                if (!$scope.profileSegments) {
                    $scope.profileSegments = [];
                }
                if (personaOverride.segments) {
                    $scope.profileSegments = personaOverride.segments;
                }

                if(personaOverride.profileProperties) {
                    $scope.profileProperties = angular.copy(personaOverride.profileProperties);
                }

                angular.forEach($scope.profileProperties, function (value, key, obj) {
                    if($scope.availableProfilePropertiesIndexed && $scope.availableProfilePropertiesIndexed[key] && $scope.availableProfilePropertiesIndexed[key].type == "date") {
                        $scope.watchDate("profileProperties." + key);
                    }
                });

                if(personaOverride.sessionProperties) {
                    $scope.sessionProperties = angular.copy(personaOverride.sessionProperties);
                }

                angular.forEach($scope.sessionProperties, function (value, key, obj) {
                    if($scope.availableSessionPropertiesIndexed && $scope.availableSessionPropertiesIndexed[key] && $scope.availableSessionPropertiesIndexed[key].type == "date") {
                        $scope.watchDate("sessionProperties." + key);
                    }
                });

                //Initializing google Map if needed
                if ($scope.selectedPersona != "none") {
                    if (_.size($scope.countries) == 0) {
                        $scope.getCountries();
                    }
                    if ($scope.sessionProperties.sessionAdminSubDiv1) {
                        $scope.getAdm1($scope.sessionProperties.sessionCountryCode, function () {
                            if ($scope.sessionProperties.sessionAdminSubDiv2) {
                                $scope.getAdm2($scope.sessionProperties.sessionCountryCode, $scope.sessionProperties.sessionAdminSubDiv1, function () {
                                    $scope.getCities($scope.sessionProperties.sessionCountryCode, $scope.sessionProperties.sessionAdminSubDiv1, $scope.sessionProperties.sessionAdminSubDiv2)
                                });
                            }
                            else {
                                $scope.getCities($scope.sessionProperties.sessionCountryCode, $scope.sessionProperties.sessionAdminSubDiv1, $scope.sessionProperties.sessionAdminSubDiv2);
                            }
                        });
                    }
                    else {
                        $scope.getCities($scope.sessionProperties.sessionCountryCode, $scope.sessionProperties.sessionAdminSubDiv1, $scope.sessionProperties.sessionAdminSubDiv2);
                    }
                    if ($scope.sessionProperties.hasOwnProperty("location")) {
                        $scope.marker.coords = {
                            latitude: $scope.sessionProperties.location.lat,
                            longitude: $scope.sessionProperties.location.lon
                        };
                        if ($scope.map.control.refresh) {
                            $scope.map.control.refresh({
                                latitude: $scope.sessionProperties.location.lat,
                                longitude: $scope.sessionProperties.location.lon
                            });
                        } else {
                            $scope.$watch("map.control.refresh", function(newVal, oldVal) {
                                if (newVal && !oldVal) {
                                    $scope.map.control.refresh({
                                        latitude: $scope.sessionProperties.location.lat,
                                        longitude: $scope.sessionProperties.location.lon
                                    });
                                }
                            });
                        }
                    }
                }
                $scope.locationProperties = ["sessionCountryName", "sessionAdminSubDiv1", "sessionAdminSubDiv2", "sessionCity"];
            });
        }

        window.parent.wem.registerCallbacks(contextLoaded);

        //Get personas list from context Server Using Profile Service
        profileService.search({
            condition: null,
            limit: 10,
            offset: 0,
            sortby: "itemId:asc",
            text: ""
        }, "personas").success(function (personas) {
            $scope.personas = personas;
            angular.forEach($scope.personas.list, function (value, key, obj) {
                if (value.properties.firstName) {
                    value.displayName = value.properties.firstName;
                }
                if (value.properties.lastName) {
                    if(value.displayName ) {
                        value.displayName  += (" " + value.properties.lastName);
                    } else {
                        value.displayName  = value.properties.lastName;
                    }
                }
                if(!value.displayName ) {
                    // TODO i18n this
                    value.displayName  = "Visitor";
                }
            });
            //Pushing currentProfile to the personas list
            $scope.personas.list.push({
                "itemId": "none",
                "displayName" : "none"
            });
        });

        var personaOverride = $scope.getPersonaOverride();
        if ($scope.selectedPersona != 'none') {
            window.parent.wem.loadPersonaContext($scope.selectedPersona, personaOverride.segments, personaOverride.profileProperties, personaOverride.sessionProperties);
        } else {
            window.parent.wem.loadContext(true, true);
        }
    }]);
